﻿using Infragistics;
using Infragistics.Controls.Schedules;
using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Windows;

namespace SimpleGantt
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            //DataContext = new XamGanttViewModel();

            var project = new Project
            {   
                Start = DateTime.Today,
                TaskSettings = new ProjectTaskSettings(),
                Settings = new ProjectSettings {
                    DefaultStartTime = new TimeSpan(0, 0, 0),
                    DefaultFinishTime = new TimeSpan(23, 59, 59)
                },
                IsReadOnly = true
            };

            project.SettingsResolved.IsRootTaskVisible = false;
            project.SettingsResolved.DateStringFormat = "d";
            project.TimeZoneInfoProviderResolved.LocalTimeZoneId = "UTC";
            project.SettingsResolved.MinutesPerDay = 1440;
            project.SettingsResolved.MinutesPerWeek = 10080;

            var daySettings = new DaySettings
            {
                IsWorkday = true,
                WorkingHours = new WorkingHoursCollection(new TimeRange(TimeSpan.FromHours(0), TimeSpan.FromHours(24)))
            };

            var scheduleDayOfWeek = new ScheduleDayOfWeek { DaySettings = daySettings };
            project.CalendarResolved.DaysOfWeek = new ScheduleDaysOfWeek
            {
                Monday = scheduleDayOfWeek,
                Tuesday = scheduleDayOfWeek,
                Wednesday = scheduleDayOfWeek,
                Thursday = scheduleDayOfWeek,
                Friday = scheduleDayOfWeek,
                Saturday = scheduleDayOfWeek,
                Sunday = scheduleDayOfWeek
            };

            var ppTask = new ProjectTask()
            {
                TaskName = "New Proposal Part 1",
                Start = DateTime.Today,
                Finish = new DateTime(2017, 9, 30)
            };

            var task1 = new ProjectTask()
            {
                TaskName = "New Time",
                Start = DateTime.Today,
                Finish = new DateTime(2017, 8, 30),
                // ManualDuration = new ManualProjectDuration(new ProjectDuration(TimeSpan.FromDays(2), ProjectDurationFormat.Days))
            };
            var task2 = new ProjectTask()
            {
                IsEstimated = true,
                TaskName = "serv1",
                Start = DateTime.Today,
                //Finish = new DateTime(2017, 5, 3),
                ManualDuration = new ManualProjectDuration(new ProjectDuration(TimeSpan.FromDays(2), ProjectDurationFormat.Days))
            };

            var ppTask2 = new ProjectTask()
            {
                TaskName = "New Proposal Part 2",
                Start = DateTime.Today,
                Finish = new DateTime(2017, 9, 30)
            };

            var task3 = new ProjectTask()
            {
                TaskName = "New Time 2",
                Start = DateTime.Today,
                Finish = new DateTime(2017, 9, 30),
                // ManualDuration = new ManualProjectDuration(new ProjectDuration(TimeSpan.FromDays(2), ProjectDurationFormat.Days))
            };
            var task4 = new ProjectTask()
            {
                IsEstimated = true,
                TaskName = "serv2",
                Start = DateTime.Today,
                //Finish = new DateTime(2017, 5, 3),
                ManualDuration = new ManualProjectDuration(new ProjectDuration(TimeSpan.FromDays(2), ProjectDurationFormat.Days))
            };

            project.RootTask.Tasks.Add(ppTask);
            ppTask.Tasks.Add(task1);
            task1.Tasks.Add(task2);

            project.RootTask.Tasks.Add(ppTask2);
            ppTask2.Tasks.Add(task3);
            task3.Tasks.Add(task4);

            DayOfWeekRecurrenceRule rule = new DayOfWeekRecurrenceRule(DayOfWeek.Tuesday);
            ProjectCalendarException pce = new ProjectCalendarException()
            {
                DateRange = new DateRange(DateTime.Today.AddDays(-1), DateTime.Today.AddDays(366)),                
                DaySettings = new DaySettings { IsWorkday = false },                
                Recurrence = new DateRecurrence { Frequency = DateRecurrenceFrequency.Daily, Count = 0 },
            };
            
            pce.Recurrence.Rules.Add(rule);

            project.CalendarResolved.Exceptions.Add(pce);
            gantt.Project = project;
            
        }

        private void Button_Click(object sender, RoutedEventArgs e)
        {
            var project = gantt.Project;

            ToggleExpandedState(project.RootTask);
            project.RootTask.IsExpanded = true;
        }

        protected void ToggleExpandedState(ProjectTask task)
        {
            task.IsExpanded ^= true;
            foreach (var childTask in task.Tasks)
                ToggleExpandedState(childTask);
        }
        
    }

    public class XamGanttViewModel : INotifyPropertyChanged
    {
        public ObservableCollection<XamGantt> Gantts
        {
            get { return new ObservableCollection<XamGantt> { new XamGantt() { Project = Project } }; }
        }

        private Project _project;
        public Project Project
        {
            get { return _project; }
            set
            {
                _project = value;
                NotifyPropertyChanged("Project");
            }
        }
        
        public XamGanttViewModel()
        {
            LoadDefaultProject();
        }

        private void LoadDefaultProject()
        {
            Project = new Project();
            Project.Start = DateTime.Today;
            var parent = new ProjectTask
            {
                TaskName = "Project Summary",
                IsManual = false,
            };

            Project.RootTask.Tasks.Add(parent);

            parent.Tasks.Add(new ProjectTask
            {
                TaskName = "Create specification and requirements",
                IsManual = false,
                Start = DateTime.Today.AddHours(8).ToUniversalTime(),
                ManualDuration = ProjectDuration.FromFormatUnits(4, ProjectDurationFormat.Days),
            });

            parent.Tasks.Add(new ProjectTask
            {
                TaskName = "Create a demo",
                IsManual = true,
                Start = DateTime.Today.AddHours(8).ToUniversalTime(),
                Duration = TimeSpan.FromHours(168),
                IsEstimated = true,
            });

            parent.Tasks.Add(new ProjectTask
            {
                TaskName = "Implementation",
                IsManual = true,
                Start = DateTime.Today.AddHours(8).ToUniversalTime(),
                DurationText = "A couple Of days",
            });

            parent.Tasks.Add(new ProjectTask
            {
                TaskName = "Testings",
                IsManual = false,
                IsActive = false,
                Start = DateTime.Today.AddHours(8).ToUniversalTime(),
                Duration = TimeSpan.FromHours(12)
            });

            parent.Tasks.Add(new ProjectTask
            {
                TaskName = "0 day Milestone",
                IsManual = false,
                Start = DateTime.Today.AddHours(8).ToUniversalTime(),
                Duration = TimeSpan.Zero
            });

            parent.Tasks.Add(new ProjectTask
            {
                TaskName = "2 day Milestone",
                IsManual = false,
                IsMilestone = true,
                Start = DateTime.Today.AddHours(8).ToUniversalTime(),
                Duration = TimeSpan.FromHours(8 * 2)
            });

            for (int i = 3; i < 50; i++)
            {
                parent.Tasks.Add(new ProjectTask
                {
                    TaskName = i.ToString() + " day Milestone",
                    IsManual = false,
                    IsMilestone = true,
                    Start = DateTime.Today.AddHours(8).ToUniversalTime(),
                    Duration = TimeSpan.FromHours(8 * i)
                });
            }
        }

        #region INotifyPropertyChanged

        public event PropertyChangedEventHandler PropertyChanged;
        protected void NotifyPropertyChanged(string propertyName)
        {
            var handler = PropertyChanged;
            if (handler != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }

        #endregion //INotifyPropertyChanged
    }
    
}
